#!/usr/bin/perl
###########################################################################
#    WWWThreads  - Interactive Discussion Forum
#    Rick Baker scream@screamingweb.com 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
###########################################################################
  use w3tvars qw(%config);
  use w3t qw($dbh);

  use strict;


# -------------
# Get the input
  my %FORM = w3t::get_input();


# ---------------
# Assign the data
  my $User       = $FORM{'Username'};
  my $OldPass    = $FORM{'OldPass'};
  my $Password   = $FORM{'Password'};
  my $Verify     = $FORM{'Verify'};
  my $Fakeemail  = $FORM{'Fakeemail'};
  my $Email      = $FORM{'Email'};
  my $Name       = $FORM{'Name'};
  my $Signature  = $FORM{'Signature'};
  my $Homepage   = $FORM{'Homepage'};
  my $Occupation = $FORM{'Occupation'};
  my $Hobbies    = $FORM{'Hobbies'};
  my $Location   = $FORM{'Location'};
  my $Bio        = $FORM{'Bio'};
  my $Sort       = $FORM{'sort_order'};
  my $Display    = $FORM{'display'};
  my $View       = $FORM{'view'};
  my $PostsPer   = $FORM{'PostsPer'};
  my $EReplies   = $FORM{'EReplies'};
  my $Totalsubs  = $FORM{'Totalsubs'};
  my $Notify     = $FORM{'Notify'};
  my $Security   = $FORM{'Security'};

# -----------------------
# Connect to the database
  w3t::db_connect;


# -----------------------------------------
# Make sure this is an admin or a moderator
  my %user = w3t::authenticate();
  if ( ($user{'Status'} ne 'Administrator') && ($user{'Status'} ne 'Moderator')
){

    w3t::not_right ("You must be logged in, and be a valid administrator
or moderator to access this.");

  }


# Make sure they set the security level between 1 and 100
  if ( ($Security < 1) || ($Security > 100) ) {
    w3t::not_right("Security level must be between 1 and 100.");
  }

# Make sure they are not setting the security level to something higher
# than their own
  if ( $user{'Security'} < $Security ) {
    w3t::not_right("You cannot set this user's security to a level higher than your own, which is $user{'Security'}.");
  }

# Get the current profile for this username
  my $Username_q = $dbh -> quote($User);
  my $query = qq!
    SELECT Status
    FROM Users
    WHERE Username = $Username_q
  !;
  my $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";


# --------------------------
# Assign the retrieved data
  my($Status) = $sth -> fetchrow_array;


# ------------------------------------------------------------------------
# If this user is a moderator we have to make sure they are only looking at
# someone with User status
  if ( ($user{'Status'} eq 'Moderator') && ($Status ne 'User') ) {
    w3t::not_right("As a moderator you cannot view the profiles of other moderators or Admins");
  }  

# --------------------------------------------------------
# If the password isn't the proper length we can't proceed
  if((length($Password)<4) || (length($Password)>20)){

    w3t::not_right("Password needs to be between 4 and 20 characters.");

  }

# -----------------------------------------------------
# If homepage is greater than 150 then we can't proceed
  if ( length($Homepage) > 150 ) {

    w3t::not_right("The Homepage entry is too long");

  }

# --------------------------------------------------------
# If Occupation is greater than 150 then we can't proceed
  if ( length($Occupation) > 150 ) {

    w3t::not_right("The Occupation entry is too long");

  }


# ----------------------------------------------------
# If hobbies is greater than 200 then we can't proceed
  if ( length($Hobbies) > 200 ) {

    w3t::not_right("The Hobbies entry is too long");

  }


# -----------------------------------------------------
# If location is greater than 200 then we can't proceed
  if ( length($Location) > 200 ) {

    w3t::not_right("The Location entry is too long");

  }


# -----------------------------------------------------------
# If the 2 given password do not match then we can't proceed
  if($Password ne $Verify){

    w3t::not_right("Passwords do not match");

  }

# --------------------------------------------------------------
# If Posts per is less than 1 or greater than 999, can't proceed
  if( ($PostsPer < 1) || ($PostsPer > 999) ) {

    w3t::not_right("Posts per page value cannot be used.");

  }


# -----------------------------------------------
# If this is a new password we need to encrypt it
  if ( $OldPass ne $Password ) {
    my $a = time();
    srand($a);
    my @passet = ('a'..'k', 'm'..'z', 'A'..'N', 'P'..'Z', '2'..'9');
    my $one   = int(rand($#passet)) + 1;
    my $two   = int(rand($#passet)) + 1;
    my $salt  = "@passet[$one]"."@passet[$two]";
    $Password = crypt($Password,$salt); 
  }

# -----------------------
# Format the query words
  my $Password_q   = $dbh -> quote($Password);
  my $Email_q      = $dbh -> quote($Email);
  my $Fakeemail_q  = $dbh -> quote($Fakeemail);
  my $Name_q       = $dbh -> quote($Name);
  my $Signature_q  = $dbh -> quote($Signature);
  my $Homepage_q   = $dbh -> quote($Homepage);
  my $Occupation_q = $dbh -> quote($Occupation);
  my $Hobbies_q    = $dbh -> quote($Hobbies);
  my $Location_q   = $dbh -> quote($Location);
  my $Bio_q        = $dbh -> quote($Bio);
  my $Username_q   = $dbh -> quote($User);
  my $Display_q    = $dbh -> quote($Display);
  my $View_q       = $dbh -> quote($View);
  my $EReplies_q   = $dbh -> quote($EReplies);
  my $Notify_q     = $dbh -> quote($Notify);


# --------------------------
# Update the User's profile
  my $query =qq!
    UPDATE Users
    SET Password   = $Password_q,
    Email          = $Email_q,
    Fakeemail      = $Fakeemail_q,
    Name           = $Name_q,
    Signature      = $Signature_q,
    Homepage       = $Homepage_q,
    Occupation     = $Occupation_q,
    Hobbies        = $Hobbies_q,
    Location       = $Location_q,
    Bio            = $Bio_q,
    Sort           = $Sort,
    Display        = $Display_q,
    View           = $View_q,
    PostsPer       = $PostsPer,
    EReplies       = $EReplies_q,
    Notify         = $Notify_q,
    Security       = $Security
    WHERE Username = $Username_q
  !;
  $dbh -> do ($query) or die "Can't execute query: $query. Reason: $DBI::errstr";


# -----------------------------------------------------------------
# Now lets see if they are subscribing or unsubscribing to anything
  for (my $i=0; $i<$Totalsubs; $i++){

    my $Subscribes = "Board$i";
    my $Board = $FORM{$Subscribes};
    my ($Board,$Sub) = split("--SUB--",$Board);

  # --------------------------------------------------------------
  # If the board is unmarked then we need to unsubscribe this user
    if ($Sub eq "NO") {
      my $Board_q    = $dbh -> quote($Board);
      my $Username_q = $dbh -> quote($User);
      my $query = qq!
        DELETE FROM Subscribe
        WHERE Username = $Username_q AND Board = $Board_q
      !;
      $dbh -> do ($query) or die "Can't execute query: $query. Reason:$DBI::errstr";

    }
  # ---------------------------------------------------------
  # If they board is marked then we need to subscribe the user
  # but we need to make sure they aren't already subscribed
    else {

      my $Board_q    = $dbh -> quote($Board);
      my $Username_q = $dbh -> quote($User);
      my $query = qq!
        SELECT *
        FROM Subscribe
        WHERE Username = $Username_q AND Board = $Board_q
      !;
      my $sth = $dbh -> prepare($query) or die "Query syntax error: $query. Reason: $DBI::errstr";
      $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
      my $rows = $sth -> rows;

    # ---------------------------------------------------------
    # They were not already subscribed so we now subscribe them
      if ($rows < 1) {

      # ----------------------------------------
      # Find out what the current post number is
        my $query = qq!
          SELECT Total
          FROM Boards
          WHERE Keyword = $Board_q
        !;
        my $sth = $dbh -> prepare($query) or die "Query syntax error: $query. Reason: $DBI::errstr";
        $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
        my ($Last) = $sth -> fetchrow_array;

      # ---------------------
      # Now we subscribe them
        my $query = qq!
          INSERT INTO Subscribe
          (Username,Board,Last)
          VALUES ($Username_q,$Board_q,$Last)
        !;
        $dbh -> do ($query) or die "Can't execute query: $query. Reason: $DBI::errstr";

      }

    }

  }

# ------------------------
# Send them a page 
  w3t::send_header ("Profile Changed","<META HTTP-EQUIV=\"Refresh\" content=\"5;url=$config{'cgiurl'}/admin/login.pl\">");
  w3t::table_header("Profile Changed");

  print "You have succesfully changed the profile.  You will now be returned to the main admin page.";


# ----------------
# send the footer
  $sth -> finish;
  w3t::send_footer();
