#!/usr/bin/perl 
###########################################################################
#    WWWThreads  - Interactive Discussion Forum
#    Rick Baker scream@screamingweb.com 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
###########################################################################
  use w3tvars qw(%config);
  use w3t qw($dbh);
  use strict;


# ----------------------
# Connect to the database
  w3t::db_connect();


# -------------
# Grab the input
  my %FORM = w3t::get_input();


# --------------------
# Assign the variables
  my $Number      = $FORM{'Number'};
  my $Title       = $FORM{'Title'};
  my $Description = $FORM{'Description'};
  my $HTML        = $FORM{'HTML'};
  my $Markup      = $FORM{'Markup'};
  my $Anonymous   = $FORM{'Anonymous'};
  my $Locked      = $FORM{'Locked'};
  my $Moderator   = $FORM{'Moderator'};
  my $Expires     = $FORM{'Expires'};
  my $Security    = $FORM{'Security'};
  my $Category    = $FORM{'Category'};


# -----------------------------------------
# Make sure this is an admin or a moderator
  my %user = w3t::authenticate();
  if ( ($user{'Status'} ne 'Administrator') && ($user{'Status'} ne 'Moderator') ){

    w3t::not_right ("You must be logged in, and be a valid administrator or moderator to access this.");

  }

# -----------------------------------------
# Make sure we have a proper security level
  if ( ($Security < 1) || ($Security > 100) ){
    w3t::not_right("Security level must be between 1 and 100.");
  }

# -------------------------------------------------------------------------
# Make sure they are not setting the security level to anything higher than
# their own
  if ( ($Security > $user{'Security'}) ) {
    w3t::not_right("You cannot set the security level to a value higher than your own, which is $user{'Security'}");
  }

# -----------------------------------------------------------------------
# If this is a moderator then we need to make sure they are the moderator
# for this board.
  if ( ($user{'Status'} eq 'Moderator') && ($user{'Username'} ne $Moderator) ) {

    w3t::not_right ("While you do have moderator privileges, you are not the moderator for this forum so you cannot edit it.");
  
  }

# -------------------------------
# Grab the keyword for this board
  my $query = qq!
    SELECT Keyword
    FROM Boards
    WHERE Number = $Number
  !;
  my $sth = $dbh -> prepare($query) or die "Can't prepare $query.  Reason: $DBI::errstr.";
  $sth -> execute() or die "Can't execute $query.  Reason: $DBI::errstr.";
  my ($Keyword) = $sth -> fetchrow_array;


# ----------------
# Update the board
  my $Title_q       = $dbh -> quote($Title);
  my $Description_q = $dbh -> quote($Description);
  my $HTML_q        = $dbh -> quote($HTML);
  my $Markup_q      = $dbh -> quote($Markup);
  my $Anonymous_q   = $dbh -> quote($Anonymous);
  my $Locked_q      = $dbh -> quote($Locked);

  my $query = qq!
    UPDATE Boards
    SET Title=$Title_q, Description=$Description_q, HTML=$HTML_q, Anonymous=$Anonymous_q, Locked=$Locked_q, Expire = $Expires, Markup = $Markup_q, Security = $Security
    WHERE Number = $Number
  !;
  $dbh -> do($query) or die "Can't execute query: $query.  Reason: $DBI::errstr";


# ----------------------------------
# Update it into the proper category
  my ($Cat,$Catnum) = split(/-PARSER-/,$Category);
  my $Category_q = $dbh -> quote($Cat);
  my $Keyword_q  = $dbh -> quote($Keyword);
  $query = qq!
    DELETE FROM Category WHERE Keyword = $Keyword_q
  !;
  $dbh -> do($query) or die "Can't execute query: $query.  Reason: $DBI::errstr";

  $query = qq!
    INSERT INTO Category (Title,Number,Keyword)
    VALUES ($Category_q,$Catnum,$Keyword_q)
  !;
  $dbh->do ($query) or w3t::not_right("That keyword is already being used.  Please try again.");


# ------------------------
# Send them a confirmation
  w3t::send_header ("The board has been changed.","<META HTTP-EQUIV=\"Refresh\" content=\"5;url=$config{'cgiurl'}/admin/login.pl\">");
  w3t::table_header("The board has been changed.");

  print "The board properties have been changed.  You will now be returned to the main administration page.";

  w3t::send_footer();
