#!/usr/bin/perl 
###########################################################################
#    WWWThreads  - Interactive Discussion Forum
#    Rick Baker scream@screamingweb.com 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
###########################################################################
  use w3tvars qw(%config);
  use w3t qw($dbh);

  use strict;


# ------------------
# Get the form input
  my %FORM = w3t::get_input();


# -----------------------
# Connect to the database
  w3t::db_connect();


# ----------------------
# Grab their preferences
  my %cookie   = w3t::get_cookie();
  my $Username = $cookie{'Username'};
  my $Password = $cookie{'Password'};
  my %user     = w3t::authenticate($Username,$Password);

  my $TextCols = $user{'TextCols'};
  my $TextRows = $user{'TextRows'};
  $TextCols ||= "80";
  $TextRows ||= "5";


# --------------------
# Assign the variables
  my $Username = $user{'Username'};
  my $Board    = $FORM{'Board'};
  my $page     = $FORM{'page'};
  my $view     = $FORM{'view'};
  my $what     = $FORM{'what'};
  my $Number   = $FORM{'Number'};
  my $mode     = $FORM{'mode'};
  my $sb       = $FORM{'sb'};
  my $sth;

# ---------------------------
# Find out the security level
  my $user_security = $user{'Security'};
  $user_security  ||= $config{'anon_security'};

# --------------------
# Authenticate the user
  if (!( defined $user{'Username'} ) ) {

    w3t::not_right ("We could not authenticate your Username/Passord.");

  }


# ------------------
# Check for any bans
  w3t::check_ban($user{'Username'});

                                                
# ---------------------------------------------------------------------------
# For security purposes we need to verify that this is user made this post or 
# if they are an admin or a moderator for this board.
  my ($Status) = $user{'Status'};


# -----------------------------------
# Get the post info from the database
  my $query = qq!
    SELECT Username,Subject,Body
    FROM $Board
    WHERE Number = $Number
  !;
  $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";


# -------------------------
# Assign the retrieved data
  my ($Postedby,$Subject,$Body) = $sth -> fetchrow_array;


# ----------------------------------------------
# Grab the moderator and security for this board
  my $Board_q = $dbh -> quote($Board);
  $query = qq!
    SELECT Moderator,Security
    FROM Boards
    WHERE Keyword = $Board_q
  !;
  $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr.  Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";


# -------------------------
# Assign the retreived data
  my ($Moderator,$Security) = $sth -> fetchrow_array;

# ------------------------------------------------------------------
# Make sure they have a high enough Security level to edit this post
  if ($Security > $user_security) {
    w3t::not_right("You do not have a high enough Security level to edit this post.");
  }

# --------------------------------------------------------------------
# Lets see if the user made this post or if the User is an admin or if
# the user is the moderator for this board.
  if ( ($Username ne $Postedby) && ($Status ne "Administrator") && ( $Username ne $Moderator) ) {
    $sth -> finish;
    w3t::not_right("We could not verify that you made this post, that you have Administrator priviledges or that you are a moderator for this board.  Because of this, we cannot allow you to edit this post.");

  }


# -------------------------------------------------------------------------
# Well everything checked out, doesn't look like a hacker so let's let them
# edit the post. 
  w3t::send_header("Edit Post");


# --------------------------
# Get rid of the line breaks
  $Body =~s /<br>/\n/g;


# -----------------
# Change " to &quot;
  $Subject =~s /"/&quot;/g;
  $Body =~s /"/&quot;/g;


# Replace the <b> and <i>
  $Body =~ s/<b>/\[b\]/ig;
  $Body =~ s/<\/b>/\[\/b\]/ig;
  $Body =~ s/<i>/\[i\]/ig;
  $Body =~ s/<\/i>/\[\/i\]/ig;

# -----------------------------
# Replace the color code markup
  $Body =~s /<font color=red>/\[red\]/ig;
  $Body =~s /<\/font color=red>/\[\/red\]/ig;
  $Body =~s /<font color=green>/\[green\]/ig;
  $Body =~s /<\/font color=green>/\[\/green\]/ig;
  $Body =~s /<font color=blue>/\[blue\]/ig;
  $Body =~s /<\/font color=blue>/\[\/blue\]/ig;
  $Body =~s /<font color=orange>/\[orange\]/ig;
  $Body =~s /<\/font color=orange>/\[\/orange\]/ig;
  $Body =~s /<font color=black>/\[black\]/ig;
  $Body =~s /<\/font color=black>/\[\/black\]/ig;
  $Body =~s /<font color=white>/\[white\]/ig;
  $Body =~s /<\/font color=white>/\[\/white\]/ig;
  $Body =~s /<font color=yellow>/\[yellow\]/ig;
  $Body =~s /<\/font color=yellow>/\[\/yellow\]/ig;
  $Body =~s /<font color=purple>/\[purple\]/ig;
  $Body =~s /<\/font color=purple>/\[\/purple\]/ig;


# ----------------------------
# Replace html quote -> markup
  $Body =~ s,<blockquote><font size=1>In reply to:</font><hr><p>(.+?)<p><hr></blockquote><p>,\[quote\]$1\[/quote\],gs;

# ---------------------------
# Convert html code -> markup
  $Body =~ s,<pre>(.+?)</pre><p>,\[pre\]\n$1\[/pre\]\n,gs;

# ----------------------------
# Replace html email -> markup
  $Body =~ s,<A HREF=&quot;mailto:.*&quot;>(.*)</A>,\[email\]$1\[/email\],gs;

# ---------------------------
# Replace html url -> markup
  $Body =~ s,<A HREF=.*>(.*)</A>,\[url\]$1\[/url\],gs;

# ---------------------------------
# Replace smileys with their markup
  $Body =~ s,<img src=&quot;$config{'images'}/frown.gif&quot;>,[frown],ig;
  $Body =~ s,<img src=&quot;$config{'images'}/smile.gif&quot;>,[smile],ig;
  $Body =~ s,<img src=&quot;$config{'images'}/wink.gif&quot;>,[wink],ig;


# ----------------------------
# Replace html image -> markup
  $Body =~ s,<IMG SRC=&quot;(.*)&quot;>,\[image\]$1\[/image\],ig;


# -----------------------------------------------------
# Let's find out if they get the default preview or not.
  my $Preview = $user{'Preview'};
  $Preview  ||= $config{'Preview'};

# -----------------
# Give them the page
  w3t::table_header("Edit post");;

  print qq~ 
  <table border=0 width=100% cellspacing=0 bgcolor=$config{'tddark'}>
  <tr><td>
  You may edit anything below to change this post, or you may delete it.  If
you choose to delete the post, and there are reply posts, then Subject of the 
post will be change to "This post has been deleted".  This is done to keep the threading format intact.
   ~;
  if ($Preview eq "on") {
    print "<form method=POST action=\"$config{'cgiurl'}/previewedit.pl\">";
  }
  else {
    print "<form method=POST action=\"$config{'cgiurl'}/modifypost.pl\">";
  }

  print qq~
  <input type=hidden name=Username value="$Username">
  <input type=hidden name=Board value="$Board">
  <input type=hidden name=Number value="$Number">
  <input type=hidden name="page" value="$page">
  <input type=hidden name="view" value="$view">
  <input type=hidden name="what" value="$what">
  <input type=hidden name="mode" value="$mode">
  <input type=hidden name="sb"   value="$sb">
  <p>
  </td></tr><tr><td bgcolor=$config{'tdlight'}>
  Subject<br>
  <input type=text name=Subject value="$Subject">
  <p> 
  <textarea cols=$TextCols rows=$TextRows wrap=soft name=Body>$Body</textarea>
  <p> 
  <input type=submit name="option" value="Change this post">

  <input type=submit name="option" value="Delete this post">
  </form>
  </td></tr></table>
  ~;


# ---------------
# send the footer
  $sth -> finish;
  w3t::send_footer(); 

