#!/usr/bin/perl
###########################################################################
#    WWWThreads  - Interactive Discussion Forum
#    Rick Baker scream@screamingweb.com 
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
###########################################################################
  use w3tvars qw(%config);
  use w3t qw($dbh);
  use strict;


# -----------------
# Get the form input
  my %FORM = w3t::get_input;


# --------------------
# Assign the variables
  my $Board    = $FORM{'Board'};
  my $Number   = $FORM{'Number'};
  my $page     = $FORM{'page'};
  my $view     = $FORM{'view'};
  my $mode     = $FORM{'mode'};
  my $sb       = $FORM{'sb'};
  my $search   = $FORM{'Search'};
  my $original = '';
  my $count    = '';
  my %notread;


# ---------------------------------------------------
# Grab the cookie to mark the posts as read or unread
  my %cookie = w3t::get_cookie;
  my $b_id   = "$Board.temp";
  my $unread = $cookie{$b_id};


# -----------------------
# Connect to the database
  w3t::db_connect();


# ---------------
# Grab their prefs
  my %cookie   = w3t::get_cookie();
  my $Username = $cookie{'Username'};
  my $Password = $cookie{'Password'};
  my %user     = w3t::authenticate($Username,$Password);


# -----------------------------------------
# If mode is not set, then mode = $postlist
  if ( (!$mode) && ($user{'Display'}) ) {
 
    $mode = $user{'Display'};

  } 
  elsif ( (!$mode) && (!$user{'Display'}) ) {
  
    $mode = $config{'postlist'};

  } 


  my $Board_q = $dbh -> quote($Board);

# ------------------------------------------------------
# Let's find out what Security level we are dealing with
  my $user_security = $user{'Security'};
  $user_security  ||= $config{'anon_security'};

# ----------------------------------------------------
# Check if this board is locked and grab the Moderator
  my $query = qq!
    SELECT Locked,Moderator,Security
    FROM Boards
    WHERE Keyword = $Board_q
  !; 
  my $sth = $dbh -> prepare($query) or die "Query Syntax Error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";


  my ($locked,$moderator,$security) = $sth -> fetchrow_array;
  $sth -> finish;

# ------------------------------------------------------------------
# If this board has a higher security level, then they cannot see it
  if ($user_security < $security) {
    w3t::not_right("You do not have a high enough security level to view this board.");
  }


# -----------------------------------------------------------------------------
# Once and a while it people try to just put a number into the url, lets trap it
  if (!$Number) {
    w3t::not_right("There was a problem looking up the Post in our database.  Please try again.");
  }

# ------------------------------------------
# Grab the main post number for this thread 
  $query = qq!
    SELECT Main,Last_Post
    FROM $Board
    WHERE Number=$Number
  !;
  $sth = $dbh -> prepare ($query) or die "Query Syntax Error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";


  my ($current,$posted) = $sth -> fetchrow_array;
  $sth -> finish;

# --------------------------------------------------------------------------
# Give them a cookie marking this message as read appending it to the others
  %cookie = w3t::get_cookie;
  my $bname = "$Board.read";
  my $read = $cookie{$bname};
  my $check = ",$Number,";
  if( (!($read =~/$check/)) && ($posted > $unread) ) {

    $read = $read.",$Number,";
    w3t::set_cookie($bname,"$read",0);

  }


# ---------------------------------------------
# Check if there are any replies in this thread
  $query = qq!
    SELECT Number
    FROM $Board
    WHERE Main=$current
  !;
  $sth = $dbh -> prepare ($query) or die "Query Syntax Error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
  my $checkreplies = $sth -> rows;


# ---------------------------------------------------------------------
# If we are doing flat mode then we need to mark every post on the page
# as read
  if( ($mode eq "flat") || ($config{'postlist'} eq "flat") ){
    for (my $i = 0; $i< $checkreplies; $i++){ 
      my ($Marknumber) = $sth -> fetchrow_array;
      my $check = ",$Marknumber,";
      if( (!($read =~/$check/)) && ($posted > $unread) ) {
        $notread{$Marknumber} = "true";
        $read = $read.",$Marknumber,";
        w3t::set_cookie($bname,"$read",0);

      }

    }

  }


# -------------------------------
# Lets give the start of the page 
  w3t::send_header("$config{'title'}");

  print"<table border=0 width=100% cellspacing=0>";
  print"<tr><td colspan=2 bgcolor=$config{'tdlight'}><p align=right>";

# -----------------
# Assign the sort order  
# If dateslip is on, we sort by Last_Post, otherwise we sort by Posted
    my %sort_opt;
    my $sort_by;
    if ($config{'dateslip'} eq "off") {
      %sort_opt =  (
        1 => 'Subject DESC',
        2 => 'Subject ASC',
        3 => 'Username DESC',
        4 => 'Username ASC',
        5 => 'Posted DESC',
        6 => 'Posted ASC'
      );
      $sort_by = $sort_opt{$sb};
      if(!$sort_by) {$sort_by = "Posted DESC";}
    }
    else {
      %sort_opt =  (
        1 => 'Subject DESC',
        2 => 'Subject ASC',
        3 => 'Username DESC',
        4 => 'Username ASC',
        5 => 'Last_Post DESC',
        6 => 'Last_Post ASC'
      );   
      $sort_by = $sort_opt{$sb};
      if(!$sort_by) {$sort_by = "Last_Post DESC";}
    }
 


# ----------------------------------------------------------------------
# Grab all the main posts from the database that are on the current page
# We also need to grab one from the page before and one from the next
# page just so we can check if there needs to be a previous or next
# thread link.
  my $PostsPer = $user{'PostsPer'};
  $PostsPer ||= $config{'postsperpage'};

  my $Totalgrab = '';
  if($page < 1) {
    $Totalgrab = $PostsPer + 2;
  }else{
    my $Startat = $page *$PostsPer -1;
    my $Posts   = $PostsPer +2;
    $Totalgrab  = "$Startat, $Posts";
  }

  $query = qq!
    SELECT Number,Main,Username,Subject,Posted
    FROM $Board
    WHERE Number = Main
    ORDER BY $sort_by
    LIMIT $Totalgrab
  !;
  
  $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
  $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
  my $total = $sth -> rows;
  my $newpage = 0;


# --------------------------------------------------------------------
# Set up some variables that we will be using at the end of flat mode
  my $previouspage = '';
  my $currentpage  = '';
  my $nextpage     = '';


# ----------------------------------------------------------------
# Cycle through all the posts to see if there are previous and next
# threads
  my ( $PNumber,$OldNumber,$Main,$OldMain,$Username,$OldUsername,$Subject,$OldSubject,$Posted,$OldPosted);

  for(my $i=0;$i<$total;$i++){

    $OldNumber = $PNumber;
    $OldMain = $Main;
    $OldUsername = $Username;
    $OldSubject = $Subject;
    $OldPosted = $Posted;
    ($PNumber,$Main,$Username,$Subject,$Posted) = $sth -> fetchrow_array;



  # --------------------------------------------------------------------
  # We are checking here to see if they get a previous link button or not
    if( ( ($PNumber eq $current) && ($page ==0) && ($i > 0) && ($search ne "true") ) ||
    ( ($PNumber eq $current) && ($page !=0) && ($search ne "true") ) ){
    

      my $whichpage = '';
      if ($i == 1) { 
        $whichpage = $page -1;
      } else {
        $whichpage = $page;
      }
      if ($whichpage < 0) { $whichpage =0; }
        $previouspage = "<a href=\"$config{'cgiurl'}/showpost.pl?Board=$Board&Number=$OldNumber&page=$whichpage&view=$view&mode=$mode&sb=$sb\"><img src = \"$config{'images'}/previous.gif\" alt = \"Previous Thread\" border=0></a>";
      my $previous     = 1;

      } 
      elsif ( ($PNumber eq $current) && ($page == 0) && ($i == 0) && ($search ne "true") ) { 

        $previouspage = "<img alt=\"*\" src = \"$config{'images'}/greyprevious.gif\">";

      }
    
    print $previouspage;

  # ---------------------------------------------------------------------
  # If we are on the current thread then we give the link for all threads
  # unless they came from the search engine then we give them a link back
  # to that.
    if($PNumber eq $current){


    # --------------------------------------------------------------------
    # If we didn't come from the search engine then they get a link to all
      if ($search ne "true"){ 

        $currentpage = "<a href = \"$config{'cgiurl'}/wwwthreads.pl?action=list&Board=$Board&page=$page&view=$view&sb=$sb\"><img src = \"$config{'images'}/all.gif\" alt = \"View All Threads\" border=0></a>";


    # -----------------------------------------------------------
    # We came from the search engine so they get a link back to it.
      } 
      else {
        my $Words = $FORM{'Words'};
        my $Forum = $FORM{'Forum'};
        my $Old   = $FORM{'Old'};
        $Words    = w3t::url_encode($Words);
        $Forum    = w3t::url_encode($Forum);
        my $Match = $FORM{'Match'};
        $Match    = w3t::url_encode($Match);
        my $Searchpage = $FORM{'Searchpage'};
        my $Limit = $FORM{'Limit'};
        $currentpage =  "<a href = \"$config{'cgiurl'}/dosearch.pl?Forum=$Forum&Words=$Words&Match=$Match&Searchpage=$Searchpage&Limit=$Limit&Old=$Old\"><img src = \"$config{'images'}/all.gif\" alt = \"Return to the search engine.\" border=0></a>";

      }      

      print $currentpage;

    # --------------------------------------------------------------------
    # Now also grab the the next thread
      ($PNumber,$Main,$Username,$Subject,$Posted) = $sth -> fetchrow_array;


    # --------------------------------------------------------------------
    # If there is a next thread then give them a link to it otherwise they
    # get a greyed out image
      if(($PNumber) && ($search ne "true") ) {

        $i = $i + 2;
        my $whichpage = ''; 
        if ( ($i == $total) && ($i > $PostsPer) ) {
          $whichpage = $page + 1;
        } else {
          $whichpage = $page;
        }

        $nextpage = "<a href=\"$config{'cgiurl'}/showpost.pl?Board=$Board&Number=$PNumber&page=$whichpage&view=$view&mode=$mode&sb=$sb\"><img src = \"$config{'images'}/next.gif\" alt = \"Next Thread\" border = 0></a>";

      } 
      elsif ($search ne "true") {
    
        $nextpage = "<img alt=\"*\" src=\"$config{'images'}/greynext.gif\">";

      }

      print $nextpage;

    # -------------------------------------------------------------------
    # If we got here then we are listing the original thread starting post
    # so we set original to 1
      $original = 1;
      last;

    }

  }


# -----------------------------------------------------------------------
# If $original was not set by the previous for loop then we are showing a
# reply, so just give a link back to the list of all threads
  if (!$original){


  # --------------------------------------------------------------------
  # If we didn't come from the search engine then they get a link to all
    if ($search ne "true"){ 

      $currentpage = "<a href = \"$config{'cgiurl'}/wwwthreads.pl?action=list&Board=$Board&page=$page&view=$view&sb=$sb\"><img src = \"$config{'images'}/all.gif\" alt = \"View All Threads\" border=0></a>";


  # ------------------------------------------------------------
  # We came from the search engine so they get a link back to it.
    } 
    else {
      my $Words = $FORM{'Words'};
      my $Forum = $FORM{'Forum'}; 
      my $Old   = $FORM{'Old'};
      $Words    = w3t::url_encode($Words);
      $Forum    = w3t::url_encode($Forum);
      my $Match = $FORM{'Match'};
      $Match    = w3t::url_encode($Match);
      my $Searchpage = $FORM{'Searchpage'};
      my $Limit = $FORM{'Limit'};
      $currentpage = "<a href = \"$config{'cgiurl'}/dosearch.pl?Forum=$Forum&Words=$Words&Match=$Match&Searchpage=$Searchpage&Limit=$Limit&Old=$Old\"><img src = \"$config{'images'}/all.gif\" alt = \"Return to the search engine.\" border=0></a>";

    }      

    print $currentpage;

  }


# --------------------------------------------------------------------------
# If $checkreplies = 1 and mode doesn't equal flat then we need to give them
# a link to show entire thread
  if (($checkreplies > 1) && ($mode ne "flat") && ($config{'postlist'} ne "flat")){

    print "<a href=\"$config{'cgiurl'}/showpost.pl?Board=$Board&Number=$Number&page=$page&view=$view&mode=flat&sb=$sb\"><img src=\"$config{'images'}/flat.gif\" alt = \"Show in Flat Mode\" border=0></a>";
    print "<img alt=\"*\" src=\"$config{'images'}/greythreaded.gif\">";


# -----------------------------------------------------------------------
# Otherwise if $checkreplies < 2 and we are not doing flat mode then they
# dont get a flat or a threaded image.
  } 
  elsif ( ($checkreplies < 2) && ($mode ne "flat") ) {

    print "<img alt=\"*\" src=\"$config{'images'}/greyflat.gif\">";
    print "<img alt=\"*\" src=\"$config{'images'}/greythreaded.gif\">";


# ---------------------------------------------------
# Otherwise they geta link to switch back to threaded
  } 
  elsif ( ($checkreplies > 1) && ($mode eq "flat") && ($config{'postlist'} ne "flat")){

    print "<img alt=\"*\" src=\"$config{'images'}/greyflat.gif\">";
    print "<a href=\"$config{'cgiurl'}/showpost.pl?Board=$Board&Number=$Number&page=$page&view=$view&mode=threaded&sb=$sb\"><img src=\"$config{'images'}/threaded.gif\" alt = \"Show in Threaded Mode\" border=0></a>";

  }
  else {

    print "<img alt=\"*\" src=\"$config{'images'}/greyflat.gif\">";
    print "<img alt=\"*\" src=\"$config{'images'}/greythreaded.gif\">";

  }
  print"</td></tr></table>";
  print"<br>";

# ---------------------------------------------------
# If we are doing flat mode then show the entire thread 
  if( ($mode eq "flat") || ($config{'postlist'} eq "flat") ){


  # -------------------------------
  # Now cycle through all the posts
    $query = qq!
      SELECT Number,Posted,Username,IP,Subject,Body,File,Status
      FROM $Board
      WHERE Main = $current
      ORDER BY Number
    !;
    $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
    $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
    my $totalthread = $sth -> rows;

    for (my $i = 0;$i< $totalthread;$i++){

      my ($Number,$Posted,$Username,$IP,$Subject,$Body,$File,$Open) = $sth -> fetchrow_array;
      my $time = w3t::convert_time($Posted);

      my $EUsername = w3t::url_encode($Username);

      # ------------------------------------------------------------------
      # If we came from the search engine then we bold the search keywords
      if ($search eq "true") { 
        my @searchwords = split(/ +/,$FORM{'Words'});
        for my $i (0 .. $#searchwords) {
          $Body =~s /$searchwords[$i]/<b><i>$searchwords[$i]<\/i><\/b>/ig;
        }
      }

      my $newimage = '';
      if ( ($notread{$Number} eq "true") && ($Posted > $unread) ) {
        $newimage = "<img alt=\"new\" src=\"$config{'images'}/new.gif\">";
      }
      my $tablerows = 3;
      if ( ($config{'showip'}) && ($IP) ) {
        $tablerows++;
      }
      if ($File) {
        $tablerows++;
      }


    # ------------------------------------------
    # Set both the reply and edit buttons to off
      my $reply = "off";
      my $edit  = "off";

    # -------------------------------------------------------
    # If the board is not locked then give them a reply button
      if ( ($locked eq "Open") && ($Open ne "C") ){
        $reply = "on";
      } 
      else {

      # --------------------------------------------------------------
      # If the board is locked then we need to see if they are an admin
      # or moderator to see if they get a reply button.
        my $Username = $user{'Username'};
        if (!$Username) { $Username = " "; }
        my $Username_q = $dbh -> quote($Username);
        my $query =qq!
          SELECT Status
          FROM Users
          WHERE Username=$Username_q
        !;
        my $sti = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
        $sti -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
        my ($checkstat) = $sti -> fetchrow_array;
      

      # -------------------------------------------
      # here we check for admin or moderator status
        if (($checkstat eq "Administrator") || ($checkstat eq "Moderator")) {
          $reply = "on";
        } 

      }


    # -----------------------------------
    # lets see if they get a edit button
    if($user{'Username'}) {


      # -------------------------------------------------
      # If they own the post or they are a moderator give
      # them an edit button
      if( ($user{'Username'} eq $Username) || ($user{'Username'} eq "$moderator") ) {
        $edit = "on";


      # ----------------------------------------------------------------------
      # otherwise the only way they get an edit button is if they are an admin
      } 
      else {

        my $Username = $user{'Username'};
        if (!$Username) { $Username = " "; }
        my $Username_q = $dbh -> quote($Username);
        my $query = qq!
          SELECT Status
          FROM Users
          WHERE Username = $Username_q
        !;
        my $stj = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
        $stj -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
        my ($admin) = $stj -> fetchrow_array;
        if ($admin eq "Administrator") {
          $edit = "on";

        }

      }

    }

    my $post_format = $user{'Post_Format'};
    $post_format  ||= $config{'post_format'};

    if ($post_format eq "top") {
    print qq~ 
    <a name="Post$Number"></a>
    <table width=100% cellspacing=0 cellpadding=0 border=0>

    <tr bgcolor=$config{'tddark'}><td width=20% valign=top>Subject</td><td>$Subject $newimage</td>
    <td rowspan=$tablerows valign=bottom align=right>
    ~;
    if ($edit eq "on") {
      print "<a href=\"$config{'cgiurl'}/editpost.pl?Board=$Board&Number=$Number&page=$page&view=$view&what=showpost&mode=$mode&sb=$sb\"><img alt =\"Edit this post\" src=\"$config{'images'}/edit.gif\" border=0></a>";
    }
    if ($reply eq "on") {
      print"<a href=\"$config{'cgiurl'}/newreply.pl?Board=$Board&Number=$Number&page=$page&view=$view&what=showpost&mode=$mode&sb=$sb\"><img src=\"$config{'images'}/reply.gif\" alt = \"Reply to this message\" border=0></a>";
    }
    else {
      print "&nbsp;";
    }

    print "</td>";

    # ---------------------------------------------------------
    # If its an anonymous post, dont' give a link to the profile
    if($Username eq "Anonymous") {

      print "<tr bgcolor=$config{'tddark'}><td>Posted by</td><td>";
      print "Anonymous</td></tr>";

    } 
    else {

      print "<tr bgcolor=$config{'tddark'}><td>Posted by</td><td><a href=\"$config{'cgiurl'}/showprofile.pl?User=$EUsername&Number=$Number&Board=$Board&what=showpost&page=$page&view=$view&mode=$mode&sb=$sb\">$Username</a></td></tr>";

    }


    print"<tr bgcolor=$config{'tddark'}><td>Posted on</td><td>$time</td></tr>";

    if ( ($config{'showip'}) && $IP) {

      print "<tr bgcolor=$config{'tddark'}><td>From IP</td><td>$IP &nbsp;</td></tr>";

    }
    if ($File){
      print "<tr bgcolor=$config{'tddark'}><td>Attachment</td><td><a href=\"$config{'fileurl'}/$File\">$File</a>";
    }

    print qq~ 
      <tr><td colspan=3 bgcolor=$config{'tdlight'}>
      <p>
      <br><br>$Body
      <br><br>
      </td></tr></table>
    ~;
    }
    elsif ($post_format eq "side") {
       
      print "\n<a name=\"Post$Number\"></a>\n";
      print "<table width=\"100%\" cellspacing=0 border=0>\n";
      # ---------------------------------------------------------
      # If its an anonymous post, dont' give a link to the profile
      if($Username eq "Anonymous") {
        print "<tr><td rowspan=2 width=17% valign=top bgcolor=$config{'tddark'}>\n";
        print "Anonymous<br>\n";
      } 
      else {
        print "<tr><td width=17% rowspan=2 bgcolor=$config{'tddark'} valign=top>\n<a href=\"$config{'cgiurl'}/showprofile.pl?User=$EUsername&Number=$Number&Board=$Board&what=showpost&page=$page&view=$view&mode=$mode&sb=$sb\">$Username</a><br>\n";
      }
      print"$time<br>\n";
      if ( ($config{'showip'}) && $IP && ($IP=~/^[0123456789]/) ) {
        print "$IP<br>\n";
      }
      if ($File){
        print "<a href=\"$config{'fileurl'}/$File\">Attachment</a>\n";
      }
      
      print "</td><td rowspan=2 width=1>\n<img src=\"$config{'images'}/clear.gif\" width=1 height=1>\n"; 
      print "</td><td bgcolor=$config{'tdlight'} align=left>\n";
      print "$Subject $newimage\n";
      print "</td><td bgcolor=$config{'tdlight'} align=right>\n";
      if ($edit eq "on") {
        print "<a href=\"$config{'cgiurl'}/editpost.pl?Board=$Board&Number=$Number&page=$page&view=$view&what=showpost&mode=$mode&sb=$sb\"><img alt =\"Edit this post\" src=\"$config{'images'}/edit.gif\" border=0></a>\n";
      }
      if ($reply eq "on") {
        print"<a href=\"$config{'cgiurl'}/newreply.pl?Board=$Board&Number=$Number&page=$page&view=$view&what=showpost&mode=$mode&sb=$sb\"><img src=\"$config{'images'}/reply.gif\" alt = \"Reply to this message\" border=0></a>\n";
      }
      else {
        print "&nbsp;\n";
      }
      print "</td></tr>\n";
      print "<tr><td width=83% colspan=2 bgcolor=$config{'tdlight'}>\n";
      print "<hr width=100% size=1 noshade>\n";
      print "<p>$Body<br><br>\n</td></tr></table>\n";
    }

  print "\n\n<br>\n\n";

  }

# --------------------------------------------
# Now give them nav links at the bottom as well
  if ($checkreplies > 1) {
    print "<table cellspacing=0 border=0 width=100% bgcolor=$config{'tdlight'}>";
    print "<tr><td align=right>";
    print "$previouspage";
    print "$currentpage";
    print "$nextpage";
    print "<img alt=\"*\" src=\"$config{'images'}/greyflat.gif\">";
    print "<a href=\"$config{'cgiurl'}/showpost.pl?Board=$Board&Number=$Number&page=$page&view=$view&mode=threaded&sb=$sb\"><img src=\"$config{'images'}/threaded.gif\" alt = \"Show in Threaded Mode\" border=0></a>";
    print "</tr></td></table>";
  }

# --------------------------
# we are doing threaded mode
  } 
  else {

    my $query = qq!
      SELECT Number,Posted,Username,IP,Subject,Body,Parent,File,Status
      FROM $Board
      WHERE Number=$Number
    !;
    $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
    $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";

    my ($Number,$Posted,$Username,$IP,$Subject,$Body,$Parent,$File,$Open) = $sth -> fetchrow_array;
    my $time = w3t::convert_time($Posted);

    my $EUsername = w3t::url_encode($Username);

    # ------------------------------------------------------------------
    # If we came from the search engine then we bold the search keywords
    if ($search eq "true") { 
      my @searchwords = split(/ +/,$FORM{'Words'});
      for my $i (0 .. $#searchwords) {
        $Body =~s /$searchwords[$i]/<b><i>$searchwords[$i]<\/i><\/b>/ig;
      }
    }

    my $tablerows =3;
    if ( ($config{'showip'}) && ($IP) ) {
      $tablerows++;
    }
    if ($File) {
      $tablerows++;
    }
    
  # ------------------------------------------
  # Set both the reply and edit buttons to off
    my $reply = "off";
    my $edit  = "off";

  # --------------------------------------------
  # If the board is open, give them a reply button
    if( ($locked eq "Open") && ($Open ne "C") ){
      $reply = "on";
    }
    else {

    # ----------------------------------------------------------------
    # If the board is locked then we need to see if they are an admin
    # or moderator to see if they get a reply button.
      my $Username = $user{'Username'};
      if(!$Username) { $Username = " "; }
      my $Username_q = $dbh -> quote($Username);
      my $query = qq!
        SELECT Status
        FROM Users
        WHERE Username = $Username_q
      !;
      $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Quer: $query";
      $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
      my ($checkstat) = $sth -> fetchrow_array;
    
    
    # -----------------------------------------
    # Check if they are a moderator or an admin
      if (($checkstat eq "Administrator") || ($checkstat eq "Moderator")) {
        $reply = "on";
      }

    }
    # --------------------------------------------------------
    # lets see if they get a edit button
    if($user{'Username'}) {

      if( ($user{'Username'} eq $Username) || ($user{'Username'} eq "$moderator") ) {
        $edit = "on";

      } 
      else {

        my $Username = $user{'Username'};
        if (!$Username) { $Username = " "; }
        my $Username_q = $dbh -> quote($Username);
        my $query =qq!
          SELECT Status
          FROM Users
          WHERE Username = $Username_q
        !;
        $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
        $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
        my ($admin) = $sth -> fetchrow_array;
        if ($admin eq "Administrator") {
          $edit = "on";
        }

      }

    }
    my $post_format = $user{'Post_Format'};
    $post_format  ||= $config{'post_format'};

    if ($post_format eq "top") {
    print qq~ 
    <a name="Post$Number"></a>
    <table width=100% cellpadding=0 cellspacing=0 border=0>

    <tr bgcolor=$config{'tddark'}><td width=20% valign=top>Subject</td><td>$Subject</td>
    <td rowspan=$tablerows valign=bottom align=right>
    ~;

    if ($edit eq "on") {
      print "<a href=\"$config{'cgiurl'}/editpost.pl?Board=$Board&Number=$Number&page=$page&view=$view&what=showpost&mode=$mode&sb=$sb\"><img alt =\"Edit this post\" src=\"$config{'images'}/edit.gif\" border=0></a>";
    }
    if ($reply eq "on") {
      print"<a href=\"$config{'cgiurl'}/newreply.pl?Board=$Board&Number=$Number&page=$page&view=$view&what=showpost&mode=$mode&sb=$sb\"><img src=\"$config{'images'}/reply.gif\" alt = \"Reply to this message\" border=0></a>";
    }
    else {
      print "&nbsp;";
    }

    print "</td></tr>";


    if ($Username eq "Anonymous"){

      print "<tr bgcolor=$config{'tddark'}><td>Posted by</td><td>";
      print "Anonymous</td></tr>";

    } else {

      print "<tr bgcolor=$config{'tddark'}><td>Posted by</td><td><a href=\"$config{'cgiurl'}/showprofile.pl?User=$EUsername&Number=$Number&Board=$Board&what=showpost&page=$page&view=$view&mode=$mode&sb=$sb\">$Username</a></td></tr>";

    }

    print"<tr bgcolor=$config{'tddark'}><td>Posted on</td><td>$time</td></tr>";

    if ( ($config{'showip'}) && ($IP) ){

      print "<tr bgcolor=$config{'tddark'}><td>From IP</td><td>$IP &nbsp;</td></tr>";

    }
    if ($File){
      print "<tr bgcolor=$config{'tddark'}><td>Attachment</td><td><a href=\"$config{'fileurl'}/$File\">$File</a>";
    }

    print qq~ 
    <tr><td colspan=3 bgcolor=$config{'tdlight'}>
    <p><br><br>$Body
    <br><br>
    </td></tr></table>
    ~;

    }
    elsif ($post_format eq "side") {
       
      print"<a name=\"Post$Number\"></a>";
      print"<table width=100% cellspacing = 0 border=0>";
      # ---------------------------------------------------------
      # If its an anonymous post, dont' give a link to the profile
      if($Username eq "Anonymous") {
        print "<tr><td rowspan=2 width=17% valign=top bgcolor=$config{'tddark'}>";
        print "Anonymous<br>";
      } 
      else {
        print "<tr><td width=17% rowspan=2 bgcolor=$config{'tddark'} valign=top><a href=\"$config{'cgiurl'}/showprofile.pl?User=$EUsername&Number=$Number&Board=$Board&what=showpost&page=$page&view=$view&mode=$mode&sb=$sb\">$Username</a><br>";

      }
      print"$time<br>";
      if ( ($config{'showip'}) && $IP && ($IP=~/^[0123456789]/) ) {
        print "$IP<br>";
      }
      if ($File){
        print "<a href=\"$config{'fileurl'}/$File\">Attachment</a>";
      }
      
      print "</td><td rowspan=2 width=1><img src=\"$config{'images'}/clear.gif\"width=1 height=1>"; 
      print "</td><td bgcolor=$config{'tdlight'} align=left>";
      print "$Subject";
      print "</td><td bgcolor=$config{'tdlight'} align=right>";
      if ($edit eq "on") {
        print "<a href=\"$config{'cgiurl'}/editpost.pl?Board=$Board&Number=$Number&page=$page&view=$view&what=showpost&mode=$mode&sb=$sb\"><img alt =\"Edit this post\" src=\"$config{'images'}/edit.gif\" border=0></a>";
      }
      if ($reply eq "on") {
        print"<a href=\"$config{'cgiurl'}/newreply.pl?Board=$Board&Number=$Number&page=$page&view=$view&what=showpost&mode=$mode&sb=$sb\"><img src=\"$config{'images'}/reply.gif\" alt = \"Reply to this message\" border=0></a>";
      }
      else {
        print "&nbsp;";
      }
      print "</td></tr>";
      print "<tr><td colspan=2 bgcolor=$config{'tdlight'} width=83%>";
      print "<hr size=1 noshade>";
      print "<p>$Body<br><br></td></tr></table>";
    }

  # -------------------------------------------------
  # Lets see if there are any replies to this message
    if($checkreplies > 1) {

      print qq~ 
      <table cellpading=0 cellspacing=0 border=0 width=100% bgcolor=$config{'tdalt'}>
      <tr><td><img alt=\"-\" src="$config{'images'}/hr.gif"></td></tr>
      </table>

      <TABLE BORDER=0 WIDTH=100% cellspacing=0>
      <tr><td align=center colspan=3 bgcolor=$config{'tddark'}><b>Entire Thread</b></center></td></tr>
      <tr bgcolor=$config{'tddark'}><td width=50%>Subject</td><td width=20%>&nbsp;&nbsp;Posted by</td><td width=30%>Posted On</td></tr>

      ~;

      my $color = $config{'tdlight'};
      my $indent = 0;


    # -----------------------------------
    # List the first post for this thread
      my $query =qq!
        SELECT Number,Parent,Posted,Username,Subject,Status
        FROM $Board
        WHERE Number = $current
      !;

      my $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
      $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";

      my ($PNumber,$Parent,$Posted,$Username,$Subject,$Locked) = $sth -> fetchrow_array;

      print "<tr bgcolor=$color><td>";

      my $time = w3t::convert_time($Posted);
      my $folder = "closedfolder.gif";
      my $thisone = ",$PNumber,";
      my $alt     = ".";
      if( ($Posted > $unread) && (!($read =~/$thisone/)) ){

        $alt    = "*";
        $folder = "newclosedfolder.gif";

      } 
      else {

        $alt    = ".";
        $folder = "closedfolder.gif";

      }

      if ($Locked eq "C") { 
        $alt    = "-";
        $folder = "lock.gif"; 
      }

      if( $PNumber == $Number ) {
        print "<img src=\"$config{'images'}/$folder\" alt =$alt hspace=5><b>$Subject</b>";
      }
      else {

        print"<a href=\"$config{'cgiurl'}/showpost.pl?Board=$Board&Number=$PNumber&page=$page&view=$view&mode=$mode&sb=$sb#Post$PNumber\"><img src=\"$config{'images'}/$folder\" alt=\"*\" border=0 hspace=5>$Subject</a>";

      }

      if ($Username eq "Anonymous") {
        print"</td><td width=150 valign=top>&nbsp;&nbsp;Anonymous</td>";
      }else{
        my $EUsername = w3t::url_encode($Username);
        print"</td><td width=150 valign=top>&nbsp;&nbsp;<a href=\"$config{'cgiurl'}/showprofile.pl?User=$EUsername&Number=$Number&Board=$Board&what=showpost&page=$page&view=$view&mode=$mode&sb=$sb\">$Username</a></td>";
      }

      print "<td valign=top>$time</td></tr>";

    # alternate the colors
      if ($color eq "$config{'tdlight'}") {
        $color = "$config{'tddark'}";
      } else {
        $color = "$config{'tdlight'}";
      }

      $color = &show_replies($Board,$current,$Number,$page,$view,$sb,$indent,$color,$unread,$mode);
    
      }

    }

# -------------------
# Give the jumper box
  $sth -> finish;
  print "</table>";
  print "<table cellpadding=0 cellspacing=0 border=0 width=100%><tr><td align=right>";

  my $query  = qq!
    SELECT Number,Title,Keyword,Security
    FROM Boards
    WHERE Security <= $user_security
    ORDER BY Title
  !;
  $sth = $dbh -> prepare($query) or die "Can't prepare $query.  Reason: $DBI::errstr.";
  $sth -> execute() or die "Can't execute $query.  Reason: $DBI::errstr.";

  print "<FORM METHOD=POST action =\"$config{'cgiurl'}/jumper.pl\">";
  print "Jump to ";
  print "<SELECT NAME=board>";
  while (my (@varlist) = $sth -> fetchrow_array) {
    print "<option value=\"$varlist[2]\">$varlist[1]\n";
  }
  print "</select>";
  print "<input type=submit name=Jump value=Jump>";
  print "</form>";
  print "</td></tr>";
  print "</table>";
  

# ----------------
# Send the footer
  w3t::send_footer;


#######################################################################
# Show replies
#######################################################################

  sub show_replies {

    my ($Board,$current,$Number,$page,$view,$sb,$indent,$color,$unread,$mode) = @_;

    my $rname  = "$Board.read";
    my %cookie = w3t::get_cookie();
    my $read   = $cookie{$rname};


  # ---------------------------------------
  # Heres where we actually list the replies
    my $query =qq!
      SELECT Number,Parent,Posted,Username,Subject,Status
      FROM $Board
      WHERE Parent = $current
      ORDER BY Number DESC
    !;

    my $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
    $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
    my $followups = $sth -> rows;

    $indent++;

    for(my $x=0;$x<$followups;$x++){

      my ($PNumber,$Parent,$Posted,$Username,$Subject,$Open) = $sth -> fetchrow_array;

      print "<tr bgcolor=$color><td>";
      for (my $y=0;$y<$indent;$y++){

        print "<img alt=\".\" src=\"$config{'images'}/blank.gif\">";

      }

      my $time = w3t::convert_time($Posted);
      my $folder = "closedfolder.gif";
      my $thisone = ",$PNumber,";
      my $alt     = ".";

      if( ($Posted > $unread) && (!($read =~/$thisone/)) ){

        $alt    = "*";
        $folder = "newclosedfolder.gif";

      } 
      else {

        $alt    = ".";
        $folder = "closedfolder.gif";

      }

      if ($PNumber == $Number) { 

        $alt    = ".";
        $folder = "closedfolder.gif"; 

      }
      if ($Open eq "C") { 
  
        $alt    = "-";
        $folder = "lock.gif"; 

      }


      if( $PNumber == $Number ) {
        print "<img src=\"$config{'images'}/$folder\" alt =$alt hspace=5><b>$Subject</b>";
      }
      else {

        print"<a href=\"$config{'cgiurl'}/showpost.pl?Board=$Board&Number=$PNumber&page=$page&view=$view&mode=$mode&sb=$sb#Post$PNumber\"><img src=\"$config{'images'}/$folder\" alt=\"*\" border=0 hspace=5>$Subject</a>";

      }

      if ($Username eq "Anonymous") {
        print"</td><td width=20% valign=top>&nbsp;&nbsp;Anonymous</td>";
      }else{
        my $EUsername = w3t::url_encode($Username);
        print"</td><td width=20% valign=top>&nbsp;&nbsp;<a href=\"$config{'cgiurl'}/showprofile.pl?User=$EUsername&Number=$Number&Board=$Board&what=showpost&page=$page&view=$view&mode=$mode&sb=$sb\">$Username</a></td>";
      }

      print "<td valign=top>$time</td></tr>";

    # alternate the colors
      if ($color eq "$config{'tdlight'}") {
        $color = "$config{'tddark'}";
      } else {
        $color = "$config{'tdlight'}";
      }

      my $query = qq!
        SELECT Number
        FROM $Board
        WHERE Parent = $PNumber
      !;
      my $sth = $dbh -> prepare ($query) or die "Query syntax error: $DBI::errstr. Query: $query";
      $sth -> execute() or die "Can't execute query: $query. Reason: $DBI::errstr";
      my $follows = $sth -> rows;
      if ($follows > 0) {

        $color = &show_replies($Board,$PNumber,$Number,$page,$view,$sb,$indent,$color,$unread,$mode);

      }

    }

    $indent--;
    $sth -> finish;
    return $color;
  }
